/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package de.eppleton.physics.editor;

import de.eppleton.physics.editor.assistant.AssistantModel;
import de.eppleton.physics.editor.assistant.AssistantView;
import de.eppleton.physics.editor.assistant.ModelHelper;
import de.eppleton.physics.editor.palette.Box2DPaletteController;
import de.eppleton.physics.editor.scene.WorldEditorScene;
import java.awt.BorderLayout;
import java.util.Collection;
import java.util.logging.Logger;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import org.jbox2d.dynamics.World;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.netbeans.spi.palette.PaletteController;
import org.openide.awt.UndoRedo;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerUtils;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;
import org.openide.windows.TopComponent;

@MultiViewElement.Registration(
    displayName = "#LBL_Box2D_VISUAL",
iconBase = "de/eppleton/jbox2d/persistence/box2d_logo.png",
mimeType = "text/x-box2d+xml",
persistenceType = TopComponent.PERSISTENCE_NEVER,
preferredID = "Box2DVisual",
position = 2000)
@NbBundle.Messages("LBL_Box2D_VISUAL=Visual")
public class Box2DWorldEditor extends javax.swing.JPanel implements MultiViewElement {
    private static Logger LOGGER = Logger.getLogger(Box2DWorldEditor.class.getName());
    private transient de.eppleton.physics.editor.Box2DDataObject obj;
    private transient JToolBar toolbar = new JToolBar();
    private transient MultiViewElementCallback callback;
    private transient final JScrollPane jScrollPane = new JScrollPane();
    private transient WorldEditorScene scene;
    private transient final String name;
    private transient PaletteController paletteController;
    private transient ExplorerManager em = new ExplorerManager();
    private final AssistantModel localmodel;
    private World world;
 

    /**
     * Creates new form Box2DWorldEditor
     */
    public Box2DWorldEditor(Lookup lkp) {
        obj = lkp.lookup(de.eppleton.physics.editor.Box2DDataObject.class);
        assert obj != null;
        name = obj.getPrimaryFile().getName();
        paletteController = Box2DPaletteController.createPalette();
        setLayout(new BorderLayout());
        add(jScrollPane, BorderLayout.CENTER);
        localmodel = ModelHelper.returnAssistantModel();
        localmodel.setContext("started");
        add(new AssistantView(localmodel), BorderLayout.NORTH);
        world = obj.getWorld();
        scene = new WorldEditorScene(em,world);
        scene.addPropertyChangeListener(obj);
        jScrollPane.setViewportView(scene.createView());

    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables

    @Override
    public String getName() {
        return "Box2DVisualElement";
    }

    @Override
    public JComponent getVisualRepresentation() {
        return this;
    }

    @Override
    public JComponent getToolbarRepresentation() {
        toolbar.removeAll();
        toolbar.add(scene.getZoomComboBox());
        Lookup actionLookup = Lookups.forPath("Box2DSceneActions");
        Collection<? extends Action> actions = actionLookup.lookupAll(Action.class);
        for (Action action : actions) {
            toolbar.add(action);
        }
        return toolbar;
    }

    public Action[] getActions() {
        Action[] retValue;
        // the multiviewObserver was passed to the element in setMultiViewCallback() method.
        if (callback != null) {
            retValue = callback.createDefaultActions();


        } else {
            // fallback..
            retValue = new Action[0];
        }
        return retValue;
    }

    @Override
    public Lookup getLookup() {
        return new ProxyLookup(Lookups.fixed(paletteController, scene), ExplorerUtils.createLookup(em, getActionMap()));
    }

    @Override
    public void componentOpened() {
    }

    @Override
    public void componentClosed() {
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
        World currentWorld = obj.getWorld();
       if(world != currentWorld){
           world = currentWorld;
           scene.setWorld(currentWorld);
       }
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return UndoRedo.NONE;
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback callback) {
        this.callback = callback;
        callback.updateTitle(name);
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }

    @Override
    public void componentShowing() {
    }

    
  
}
